library hydrofoil;

import 'package:flutter/material.dart';
import 'package:flutter_boost/flutter_boost.dart';
import 'package:hydrofoil/channel.dart';

class Hydrofoil {
  static final Hydrofoil _instance = Hydrofoil();
  static Hydrofoil get singleton => _instance;

  Channel _channel;
  Channel get channel => _channel;

  Hydrofoil();

  /// Register a map builders
  void register({
    String channelName = 'flutter_native_channel', //todo 是否需要channelName字段
    @required Map<String, PageBuilder> builders,
  }) {
    print(channelName);
    _channel = Channel(channelName);
    FlutterBoost.singleton.registerPageBuilders(builders);
  }

  /// Register callbacks for Navigators push & pop
  void addBoostNavigatorObserver(NavigatorObserver observer) =>
      ContainerNavigatorObserver.boostObservers.add(observer);

  /// 打开路由
  Future<Map<dynamic, dynamic>> open(
    String url, {
    Map<String, dynamic> urlParams,
    Map<String, dynamic> exts,
  }) {
    return FlutterBoost.singleton.open(url, urlParams: urlParams, exts: exts);
  }

  /// 关闭
  Future<bool> close(
    BuildContext context, {
    Map<String, dynamic> result,
    Map<String, dynamic> exts,
  }) {
    BoostContainerSettings settings = BoostContainer.of(context).settings;
    return FlutterBoost.singleton
        .close(settings.uniqueId, result: result, exts: exts);
  }

  Future<bool> closeTo(
    String id, {
    Map<String, dynamic> result,
    Map<String, dynamic> exts,
  }) {
    return FlutterBoost.singleton.close(id, result: result, exts: exts);
  }

  static TransitionBuilder init({
    TransitionBuilder builder,
    PrePushRoute prePush,
    PostPushRoute postPush,
  }) {
    return FlutterBoost.init(
      builder: builder,
      prePush: prePush,
      postPush: postPush,
    );
  }

  /// 往 Native 发送事件
  void sendEvent(String name, Map<String, dynamic> arguments) {
    channel.sendEvent(name, arguments); //todo 是否新增一个type
    FlutterBoost.singleton.channel.sendEvent(name, arguments);
  }

  /// 监听 Native 发送过来的事件
  void addEventListener(String name, EventListener listener) {
    channel.addEventListener(name, listener);
    FlutterBoost.singleton.channel.addEventListener(name, listener);
  }

  /// 调用 Native 的方法
  Future<T> call<T>(
    String method, {
    Map<String, dynamic> params,
  }) {
    return _call(9001, method: method, params: params);
  }

  Future<T> _call<T>(
    int type, {
    String method,
    Map<String, dynamic> params,
  }) {
    var arguments =
        _generateArguments(type: type, method: method, params: params);
    return _invokeMethod('flutterInvokeNative', arguments);
  }

  Map<String, dynamic> _generateArguments({
    @required int type,
    String method,
    Map<String, dynamic> params,
  }) {
    Map<String, dynamic> arguments = Map<String, dynamic>();
    arguments['type'] = type;
    arguments['method'] = method;
    arguments['params'] = params;
    return arguments;
  }

  Future<T> _invokeMethod<T>(
    String method, [
    Map<String, dynamic> arguments,
  ]) async {
    assert(method != "__event__");

    return channel.invokeMethod<T>(method, arguments);
  }
}

extension HydrofoilImpl on Hydrofoil {
  // todo open /close 等

  Future<Map<K, V>> getWifi<K, V>() {
    return _call(8001);
  }

  Future<Map<dynamic, dynamic>> getBaterry(Map<String, dynamic> params) {
    return _call(8002, params: params);
  }
}
